@@ -888,13 +888,55 @@ class MolecularComposition(MoleculeBase, ResidueBase):  # pragma: no cover
         return not (self == other)
 
 
-_CompositionBase = dict
+class _CompositionBase(dict):
 
+    def _setitem_fast(self, key, value):
+        dict.__setitem__(self, key, value)
+
+    def _getitem_fast(self, key):
+        try:
+            return dict.__getitem__(self, key)
+        except KeyError:
+            return 0
+
+    def __reduce__(self):
+        return self.__class__, (), self.__getstate__()
+
+    def __getstate__(self):
+        d = {
+            'mapping': dict(self),
+            'reducing_end': self._reducing_end,
+            'composition_offset': self._composition_offset
+        }
+        return d
+
+    def __setstate__(self, state):
+        self.update(state['mapping'])
+        self._reducing_end = state['reducing_end']
+        self._composition_offset = state['composition_offset']
+
+    @classmethod
+    def _empty(cls):
+        inst = cls.__new__(cls)
+        inst._composition_offset = water_composition.clone()
+        inst._reducing_end = None
+        inst._mass = None
+        return inst
+
+    def _update_from_typed_map(self, template, copy_nodes=False):
+        if copy_nodes:
+            for name, count in template.items():
+                self._setitem_fast(name.clone(), count)
+        else:
+            for name, count in template.items():
+                self._setitem_fast(name, count)
+        reduced = template.reducing_end
+        if reduced is not None:
+            self.reducing_end = reduced.clone()
+        self._mass = None
 
 try:
-    # from glypy._c.count_table import CountTable
-    # _CompositionBase = CountTable
-    pass
+    from glypy._c.structure.glycan_composition import _CompositionBase
 except ImportError:
     pass
 
@@ -1015,9 +1057,6 @@ class GlycanComposition(_CompositionBase, SaccharideCollection):
         _CompositionBase.__setitem__(self, key, int(value))
         self._mass = None
 
-    def _setitem_fast(self, key, value):
-        _CompositionBase.__setitem__(self, key, value)
-
     def __getitem__(self, key):
         """
         Get the quantity of `key`
@@ -1042,12 +1081,6 @@ class GlycanComposition(_CompositionBase, SaccharideCollection):
         except KeyError:
             return 0
 
-    def _getitem_fast(self, key):
-        try:
-            return _CompositionBase.__getitem__(self, key)
-        except KeyError:
-            return 0
-
     def __delitem__(self, key):
         if isinstance(key, basestring):
             key = self._key_parser(key)
@@ -1342,7 +1375,7 @@ class GlycanComposition(_CompositionBase, SaccharideCollection):
 
 
     def clone(self, propogate_composition_offset=True, copy_nodes=True):
-        dup = self.__class__()
+        dup = self._empty()
         dup._update_from_typed_map(self, copy_nodes=copy_nodes)
         if not propogate_composition_offset:
             dup._composition_offset = Composition('H2O')
@@ -1350,18 +1383,6 @@ class GlycanComposition(_CompositionBase, SaccharideCollection):
             dup._composition_offset = self._composition_offset.clone()
         return dup
 
-    def _update_from_typed_map(self, template, copy_nodes=False):
-        if copy_nodes:
-            for name, count in template.items():
-                self._setitem_fast(name.clone(), count)
-        else:
-            for name, count in template.items():
-                self._setitem_fast(name, count)
-        reduced = template.reducing_end
-        if reduced is not None:
-            self.reducing_end = reduced.clone()
-        self._mass = None
-
     # inheriting from dict overwrites MoleculeBase.copy
     def copy(self, *args, **kwargs):
         return self.clone(*args, **kwargs)
@@ -1423,11 +1444,11 @@ class GlycanComposition(_CompositionBase, SaccharideCollection):
         :class:`GlycanComposition`
         """
         tokens, reduced = cls._get_parse_tokens(string)
-        inst = cls()
+        inst = cls._empty()
         deriv = None
         for token in tokens:
             try:
-                residue, count = token.split(":")
+                residue, count = _parse_name_count(token)
             except ValueError:
                 if string == "{}":
                     return inst
@@ -1438,7 +1459,7 @@ class GlycanComposition(_CompositionBase, SaccharideCollection):
                 _deriv = has_derivatization(key)
                 if _deriv:
                     deriv = _deriv
-            inst._setitem_fast(key, int(count))
+            inst._setitem_fast(key, count)
         inst._handle_reduction_and_derivatization(reduced, deriv)
         return inst
 
@@ -1507,22 +1528,23 @@ class FrozenGlycanComposition(GlycanComposition):
     @classmethod
     def parse(cls, string):
         tokens, reduced = cls._get_parse_tokens(string)
-        inst = cls()
+        inst = cls._empty()
         deriv = None
+        key_parser = cls._key_parser
         for token in tokens:
             try:
-                residue, count = token.split(":")
+                residue, count = _parse_name_count(token)
             except ValueError:
                 if string == "{}":
                     return inst
                 else:
                     raise ValueError("Malformed Token, %s" % (token,))
-            key = cls._key_parser(residue)
+            key = key_parser(residue)
             if "^" in residue:
                 _deriv = has_derivatization(key)
                 if _deriv:
                     deriv = _deriv
-            inst._setitem_fast(key, int(count))
+            inst._setitem_fast(key, count)
         inst._handle_reduction_and_derivatization(reduced, deriv)
         return inst
 
@@ -1582,7 +1604,7 @@ class FrozenGlycanComposition(GlycanComposition):
         self._total_composition = None
 
     def clone(self, propogate_composition_offset=True, copy_nodes=False):
-        dup = self.__class__()
+        dup = self._empty()
         dup._update_from_typed_map(self, copy_nodes=copy_nodes)
         if not propogate_composition_offset:
             dup._composition_offset = Composition('H2O')
@@ -1604,3 +1626,17 @@ class HashableGlycanComposition(FrozenGlycanComposition):
 
     def __hash__(self):
         return hash(str(self))
+
+
+def _parse_name_count(string):
+    name, count = string.split(":")
+    count = int(count)
+    return name, count
+
+
+try:
+    _has_c = True
+    from glypy._c.utils import get_parse_tokens, parse_name_count as _parse_name_count
+    GlycanComposition._get_parse_tokens = get_parse_tokens
+except ImportError:
+    _has_c = False
